# Figure 1: bursting Morris-Lecar neurons

using OrdinaryDiffEq
using PyPlot
using Peaks
using ProgressMeter
using RecurrenceAnalysis
using DelayEmbeddings
include("my_functions.jl")
pygui(true)

# parameters
V_E = -17.63;
A = 0.1;
dt = 0.025;
t0 = (0., 1000.);

# initial condition
y0 = [-65.;0.];

omega0 = [.02, .05, .06, .1, .286, .32, .5];
str = ["13 bursts";"4 bursts";"3 bursts";"2 bursts";"chaos";"2:3 mode locked";"1 spike"];

# results matrix
x = zeros((2,40001,length(omega0)));

# integrate model/ time series
@inbounds for (i,omega) in enumerate(omega0)
    
    par = (A, omega, V_E)
    
    # solve model
    prob = ODEProblem(ml!,y0,t0,par);
    sol = solve(prob,Tsit5(),saveat=dt, dtmax=1.0);
    x[1,:,i] = sol.t;
    x[2,:,i] = sol[1,:];
end

# plot data
fig = PyPlot.figure(figsize=(12,4))
for (i, omega) in enumerate(omega0)
   ax = fig.add_subplot(2,4,i)
   ax.plot(x[1,:,i], x[2,:,i])
   ax.axis([200, 850, -80, 80])
   plt.title(str[i])
   plt.xlabel("Time (ms)")
   plt.ylabel("Voltage (mV)")
end




# Figure 3: RPs of Morris-Lecar neurons

# parameters for RPs
j = [2 4 5 7]; # select only those time series

segment = [200 450];

# find embedding parameters
e = Array{Any}(undef,length(j));
taus = Array{Any}(undef,length(j));

for i in 1:length(j)
   e_, taus_ = pecuzal_embedding(x[2,1:3:30000,j[i]]; τs = 0:40, econ = true, verbose=false);
   e[i] = e_;
   taus[i] = taus_;
end


# calculate RPs
idx = Array{Int64}(undef,length(j),2);
X = Array{Any}(undef,length(j));

for i in 1:length(j)
   m = size(e[i],2); tau = maximum(taus[i]);
   
   idx1 = findfirst(x -> x>=segment[1], x[1,1:3:end,j[i]]);
   idx2 = findfirst(x -> x>=segment[2]+(m-1)*tau, x[1,1:3:end,j[i]]);
  
   
   idx[i,:] = [idx1, idx2];
   e_ = e[i]
   X[i] = RecurrenceMatrix(e_[idx1:idx2,:], .15, fixedrate = true);
end

# plot RPs
fig = PyPlot.figure(figsize=(5,5))
for i in 1:length(j)
   ax = fig.add_subplot(2,2,i)
   Rg = grayscale(X[i])
   t1 = x[1,idx[i,1],j[i]]
   t2 = x[1,idx[i,2],j[i]]
   ax.imshow(Rg, extent=[t1, t2, t1, t2], cmap="gray")
   plt.title(str[j[i]])
   plt.xlabel("Time (ms)")
   plt.ylabel("Time (ms)")
end


# Figure 4: tau-RR

# calculate CRQAD
taurr = Array{Any}(undef,length(j));
for i in 1:length(j)
   tau_ = zeros(size(X[i],1),1);
   for k in 1:length(tau_)
      tau_[k] = mean(diag(X[i],k));
   end
   taurr[i] = tau_;
end


# plot CRQAD
fig = PyPlot.figure(figsize=(5,4))
for i in 1:length(j)
   ax = fig.add_subplot(2,2,i)
   ax.plot(dt*(0:length(taurr[i])-1),taurr[i])
   plt.title(str[j[i]])
   plt.xlabel("Time (ms)")
   plt.ylabel("tauRR")
   plt.xlim([-0.1, 75])
end

