# Figure 6, 10, 11: frequency mismatch, CPR and Hellinger distance

using OrdinaryDiffEq
using RecurrenceAnalysis
using DelayEmbeddings
using StatsBase
using ProgressMeter
using DelimitedFiles
using Distances
using TimeseriesSurrogates
using PyPlot
include("my_functions.jl")

###### for changing coupling

# parameters
dt = 0.05
omega = .286; 
V_E = -17.63;
A = 0.1;
g = 0.1
M = 200 # number of different g tested
g0 = range(0.,stop=.15,length=M)
usePecuzal = true

# initial condition
y0 = [-65.6;0. ;-60.; 0.01];

# time
t0 = (0., 2800.);
t = 0.:dt:2800.; 

# results matrices
C = zeros(M,4);
C_ = zeros(M,4);
dphase = zeros(M,1); dfreq = zeros(M,1);
dphase_ = zeros(M,1); dfreq_ = zeros(M,1);

# calculation
p = Progress(M);
@inbounds for (i,g) in enumerate(g0)

    # solve model
    prob = ODEProblem(ml2!,y0,t0,(A, omega, V_E, g));
    prob_ = ODEProblem(ml2!,y0,t0,(0., omega, V_E, g));
    
    sol = solve(prob,Tsit5(),saveat=dt, dtmax=1.0);
    sol_ = solve(prob_,Tsit5(),saveat=dt, dtmax=1.0);

    # time series
    y1 = copy(sol[1,10001:1:50000]); y2 = copy(sol[3,10001:1:50000]);
    y1_ = copy(sol_[1,10001:1:50000]); y2_ = copy(sol_[3,10001:1:50000]);
    
    # embedding using PECUZAL (for RP analysis only)
    if usePecuzal
        e1, taus1 = pecuzal_embedding(y1[1:5:end]; τs = 0:10, econ = true, verbose=false);
        e2, taus2 = pecuzal_embedding(y2[1:5:end]; τs = 0:10, econ = true, verbose=false);
        #Plots.scatter(e1[:,1].+10,e1[:,2].+30)    
        #Plots.scatter!(e2[:,1].+10,e2[:,2].+30)    
        e1_, taus1_ = pecuzal_embedding(y1_[1:5:end]; τs = 0:10, econ = true, verbose=false);
        e2_, taus2_ = pecuzal_embedding(y2_[1:5:end]; τs = 0:10, econ = true, verbose=false);
    else
        e1 = DelayEmbeddings.embed(y1[1:5:end], 2, 5)
        e2 = DelayEmbeddings.embed(y2[1:5:end], 2, 5)
        e1_ = DelayEmbeddings.embed(y1_[1:5:end], 2, 5)
        e2_ = DelayEmbeddings.embed(y2_[1:5:end], 2, 5)
    end
    
    # CPR measure
    # providing:
    # 1 – CPR(pearson)
    # 2 - CPR(spearman)
    # 3 - Kullback-Leibler entropy
    # 4 - Hellinger distance
    C[i,1], C[i,2], C[i,3], C[i,4] = cpr(e1, e2, 0.1)
    C_[i,1], C_[i,2], C_[i,3], C_[i,4] = cpr(e1_, e2_, 0.1)
            
    # phase and frequency
    dphase[i], dfreq[i] = phasediff(y1, y2, dt)
    dphase_[i], dfreq_[i] = phasediff(y1_, y2_, dt)
    
    #f1 = frequ(y1, dt)
    #f2 = frequ(y2, dt)
   
    next!(p)
end

# Figure 6
PyPlot.plot(g0,dfreq, label = "dFreq(A>0)")
PyPlot.plot(g0,dfreq_, label = "dFreq(A=0)")
PyPlot.legend()

# Figure 10
PyPlot.clf()
PyPlot.plot(g0,C[:,1], label = "CPR^P(A>0)")
PyPlot.plot(g0,C_[:,1], label = "CPR^P(A=0)")
PyPlot.legend()



# Figure 11
# (first we prform a surrogate test for the Hellinger distance)

K = 1000 # number of surrogates for significance test
C_CI = zeros(4);
Csurr=zeros(K,4)

# solve model for the baseline
prob = ODEProblem(ml2!,y0,t0,(0., omega, V_E, .07));
sol = solve(prob,Tsit5(),saveat=dt, dtmax=1.0);
# time series
y1 = copy(sol[1,10001:1:50000]); y2 = copy(sol[3,10001:1:50000]);

# embedding using PECUZAL (for RP analysis only)
if usePecuzal
    e1, taus1 = pecuzal_embedding(y1[1:5:end]; τs = 0:10, econ = true, verbose=false);
else
    e1 = DelayEmbeddings.embed(y1[1:5:end], 2, 5)
end

# create surrogates
sg = surrogenerator(y2, BlockShuffle(5))
p = Progress(K);
@inbounds for k in range(1, stop = K) 
    s = sg()
    if usePecuzal
        e2, taus2_ = pecuzal_embedding(s[1:5:end]; τs = 0:10, econ = true, verbose=false);
    else
        e2 = DelayEmbeddings.embed(s[1:5:end], 2, 5);
    end
    Csurr[k,1], Csurr[k,2], Csurr[k,3], Csurr[k,4] = cpr(e1, e2, 0.1)
    next!(p)
end

# use 5% quantile of the Hellinger distance as the lower bound
C_CI = quantile(Csurr[:,4],.05);


PyPlot.clf()
PyPlot.plot(g0,C[:,4], label = "CPR^P(A>0)")
PyPlot.plot(g0,C_[:,4], label = "CPR^P(A=0)")
PyPlot.plot([g0[1], g0[end]],[C_CI, C_CI], label = "confidence limit")
PyPlot.legend()

       

