# Figure 8 and 9: RPs and tau-RR for coupled neurons

using OrdinaryDiffEq
using RecurrenceAnalysis
using DelayEmbeddings
using StatsBase
using ProgressMeter
using DelimitedFiles
using PyPlot
include("my_functions.jl")

###### calculate time series for coupled neurons as well as recurrence plots and tauRR

# parameters
dt = 0.05
omega = .286; 
V_E = -17.63;
A = 0.1;
g = 0.01
M = 100

# initial condition
y0 = [-65.6;0. ;-60.; 0.01];

# time
t0 = (0., 2800.);
t = 0.:dt:2800.; 

# solve model
prob = ODEProblem(ml2!,y0,t0,(A, omega, V_E, 0.01));
sol1 = solve(prob,Tsit5(),saveat=dt, dtmax=1.0);
prob = ODEProblem(ml2!,y0,t0,(A, omega, V_E, 0.04));
sol2 = solve(prob,Tsit5(),saveat=dt, dtmax=1.0);

# time series
y11 = copy(sol1[1,10001:10:50000]); y21 = copy(sol1[3,10001:10:50000]);
y12 = copy(sol2[1,10001:10:50000]); y22 = copy(sol2[3,10001:10:50000]);
t = copy(sol2.t[10001:10:50000])

# embedding using PECUZAL (for RP analysis only)
e11, taus1 = pecuzal_embedding(y11[1:1:500]; τs = 0:10, econ = true);
e21, taus2 = pecuzal_embedding(y21[1:1:500]; τs = 0:10, econ = true);
e12, taus1 = pecuzal_embedding(y12[1:1:500]; τs = 0:10, econ = true);
e22, taus2 = pecuzal_embedding(y22[1:1:500]; τs = 0:10, econ = true);


e = .1 # recurrence threshold
R11 = RecurrenceMatrix(e11, e, fixedrate = true);
R21 = RecurrenceMatrix(e21, e, fixedrate = true);
R12 = RecurrenceMatrix(e12, e, fixedrate = true);
R22 = RecurrenceMatrix(e22, e, fixedrate = true);
Rg11 = grayscale(R11)
Rg21 = grayscale(R21)
Rg12 = grayscale(R12)
Rg22 = grayscale(R22)

# tau-RR
taurr11 = RecurrenceAnalysis.tau_recurrence(R11);
taurr21 = RecurrenceAnalysis.tau_recurrence(R21);
taurr12 = RecurrenceAnalysis.tau_recurrence(R12);
taurr22 = RecurrenceAnalysis.tau_recurrence(R22);




# Figure 8: plot RPs
fig = PyPlot.figure(figsize=(5,5))
ax1 = fig.add_subplot(2,2,1)
ax1.imshow(Rg11, extent=[t[1], t[500], t[1], t[500]], cmap="gray")
plt.title("neuron 1, g = 0.01")

ax2 = fig.add_subplot(2,2,2)
ax2.imshow(Rg21, extent=[t[1], t[500], t[1], t[500]], cmap="gray")
plt.title("neuron 2, g = 0.01")

ax3 = fig.add_subplot(2,2,3)
ax3.imshow(Rg12, extent=[t[1], t[500], t[1], t[500]], cmap="gray")
plt.title("neuron 1, g = 0.04")

ax4 = fig.add_subplot(2,2,4)
ax4.imshow(Rg22, extent=[t[1], t[500], t[1], t[500]], cmap="gray")
plt.title("neuron 2, g = 0.04")


# Figure 9: plot tau-RR
# (we multiply with 10 because we downsampled time series by 10)
fig = PyPlot.figure(figsize=(10,3))
ax1 = fig.add_subplot(1,2,1)
ax1.plot(10*dt*(1:length(taurr11)),taurr11)
ax1.plot(10*dt*(1:length(taurr21)),taurr21)
plt.xlabel("Lag (ms)")
plt.ylabel("tau-RR")

ax2 = fig.add_subplot(1,2,2)
ax2.plot(10*dt*(1:length(taurr12)),taurr12)
ax2.plot(10*dt*(1:length(taurr22)),taurr22)
plt.xlabel("Lag (ms)")
plt.ylabel("tau-RR")

