# single Morris-Lecar neuron
using DSP
using Statistics
using Distances

function dv(t, A, omega, V_E)
    A/omega * sin(omega*t) + V_E
end

function dv1(t, A, omega)
    -A * cos(omega * t)
end

function ml!(dy, y, p, t)
    
    phi = 0.15;
    u_1 = -1.2;
    u_2 = 18.;
    u_3 = -13.;
    u_4 = 10.;
    e_K = -100.;
    e_leak = -70.;
    e_Na = 50.;
    g_fast = 20.;
    g_slow = 20.;
    g_leak = 2.;
    C = 2.;
    
    m_1 = 0.5 * (1. + tanh((y[1] - u_1) / u_2));
    m_2 = 0.5 * (1. + tanh((y[1] - u_3) / u_4));
    b = 1 ./ cosh((y[1] - u_3) / (2. * u_4));

    i_stim = 0.;
    i_fast = g_fast * m_1 * (y[1] + dv(t, p[1], p[2], p[3]) - e_Na);
    i_slow = g_slow * y[2] * (y[1] + dv(t, p[1], p[2], p[3]) - e_K);
    i_leak = g_leak * (y[1] + dv(t, p[1], p[2], p[3]) - e_leak);

    dy[1] = (i_stim - dv1(t, p[1], p[2]) - i_fast - i_leak - i_slow) / C;
    dy[2] = phi * (m_2 - y[2]) / b;
end



# coupled Morris-Lecar neurons
function ml2!(dy, y, p, t)

    g = p[4]
    
    phi = 0.15;
    u_11 = -1.2;
    u_21 = 18.0;
    u_31 = -12.8;
    u_41 = 10.;
    u_12 = -1.2;
    u_22 = 18.1;
    u_32 = -10.0;
    #u_32 = -4.;
    u_42 = 10.;
    e_K = -100.;
    e_leak = -70.;
    e_Na = 50.;
    g_fast = 20.;
    g_slow = 20.;
    g_leak = 2.;
    C = 2.;

    m_11 = 0.5 * (1 + tanh((y[1] - u_11) / u_21));
    m_21 = 0.5 * (1 + tanh((y[1] - u_31) / u_41));
    m_12 = 0.5 * (1 + tanh((y[3] - u_12) / u_22));
    m_22 = 0.5 * (1 + tanh((y[3] - u_32) / u_42));
    b1 = 1 ./ cosh((y[1] - u_31) / (2. * u_41));
    b2 = 1 ./ cosh((y[3] - u_32) / (2. * u_42));

    i_stim1 = 0.;
    i_fast1 = g_fast * m_11 .* (y[1] + dv(t, p[1], p[2], p[3]) - e_Na);
    i_slow1 = g_slow * y[2] .* (y[1] + dv(t, p[1], p[2], p[3]) - e_K);
    i_leak1 = g_leak * (y[1] + dv(t, p[1], p[2], p[3]) - e_leak);
    i_stim2 = 0.;
    i_fast2 = g_fast * m_12 .* (y[3] + dv(t, p[1], p[2], p[3]) - e_Na);
    i_slow2 = g_slow * y[4] .* (y[3] + dv(t, p[1], p[2], p[3]) - e_K);
    i_leak2 = g_leak * (y[3] + dv(t, p[1], p[2], p[3]) - e_leak);

    dy[1] = (i_stim1 + p[1] * cos(p[2] * t) - i_fast1 - i_leak1 - i_slow1 - g*(y[1]-y[3])) / C;
    dy[2] = phi * (m_21 - y[2]) / b1;

    dy[3] = (i_stim2 - p[1] * cos(p[2] * t) - i_fast2 - i_leak2 - i_slow2 - g*(y[3]-y[1])) / C;
    dy[4] = phi * (m_22 - y[4]) / b2;

end

# CPR measure
function cpr(x, y, e)
    N1 = length(x);
    N2 = length(y);
    N = min(N1,N2);

    # recurrence plots
    R1 = RecurrenceMatrix(x, e, fixedrate = true);
    R2 = RecurrenceMatrix(y, e, fixedrate = true);

    # tau-RR
    taurr1 = RecurrenceAnalysis.tau_recurrence(R1);
    taurr2 = RecurrenceAnalysis.tau_recurrence(R2);

    # CPR measure
    start_tau = 60
    
    CPR_ = abs(cor(taurr1[start_tau:N],taurr2[start_tau:N]))
    CPR1_ = abs(corspearman(taurr1[start_tau:N],taurr2[start_tau:N]))
    
    
    # alternative CPR measure using Kullback Leibler divergence
    taurr1 = taurr1[start_tau:N] ./ sum(taurr1[start_tau:N])
    taurr2 = taurr2[start_tau:N] ./ sum(taurr2[start_tau:N])
    idx = findall((taurr1 .> 0) .& (taurr2 .> 0))
    CPR2_ = kldivergence(taurr1[idx], taurr2[idx], length(idx)) + kldivergence(taurr2[idx], taurr1[idx], length(idx))
    #CPR3_ = get_mutual_information(taurr1[idx], taurr2[idx])/log(length(idx))
    CPR3_ = hellinger(taurr1[idx], taurr2[idx])
    
    return CPR_, CPR1_, CPR2_, CPR3_
end

# plot tauRR
function plot_tauRR(x, m, tau, e)
    N = length(x)

    # phase space embedding
    tr1 = embed(x, m, tau);

    # recurrence plots
    R1 = RecurrenceMatrix(tr1, e, scale=mean);

    # tau-RR
    taurr1 = RecurrenceAnalysis.tau_recurrence(R1);

    # plot tauRR
    start_tau = 200
    plot(taurr1[start_tau:end])
end

function phase(x::Vector{Float64})
    hx = hilbert(x);
    xR = real(hx[200:end-200]); xI = imag(hx[200:end-200]);

    if xI[1] < 0
       i1 = findfirst(x->x>0, xI)
       i2 = findlast(x->x<0, xI)
    else
       i1 = findfirst(x->x<0, xI)
       i2 = findlast(x->x>0, xI)
    end

    return unwrap(atan.(xI[i1:i2],xR[i1:i2])) ;
end

function phasediff(x::Vector{Float64}, y::Vector{Float64}, dt::Float64)
    
    hx = hilbert(x);
    x1 = real(hx[200:end-200]); x2 = imag(hx[200:end-200]);
    hy = hilbert(y);
    y1 = real(hy[200:end-200]); y2 = imag(hy[200:end-200]);
    
    if x2[1] < 0
       i1 = findfirst(x->x>0, x2)
       i2 = findlast(x->x<0, x2)
    else
       i1 = findfirst(x->x<0, x2)
       i2 = findlast(x->x>0, x2)
    end
           
    phi = [unwrap(atan.(x2[i1:i2],x1[i1:i2])) unwrap(atan.(y2[i1:i2],y1[i1:i2]))];
    
    freq1 = 2*pi*mean(diff(phi[:,1])/dt);
    freq2 = 2*pi*mean(diff(phi[:,2])/dt);
   
    # phase difference
    #dphase = mean(broadcast(abs,diff(phi,dims=2)));
    dphase = mean(diff(phi,dims=2));
    
    # frequency difference
    
    #dfrequ = atan.((x2 .* y1 .- x1 .* y2) ./ (x1 .* y1 .+ x1 .* y2))
    
    dfrequ = abs(reduce(vcat,freq1-freq2));
    
    return dphase, dfrequ
end

function frequ(x::Vector{Float64}, dt::Float64)

    hx = hilbert(x);
    x1 = real(hx);
    x2 = imag(hx);
        
    phase = unwrap(atan.(x2[100:end-100],x1[100:end-100]));
    
    freq = 2*pi*mean(diff(phase[:,1]));

    return freq
end



function evolv_phasediff(x, y, tau, dt)

    result = zeros(2,1)
    
    # phase space embedding
    tr1 = embed(x.-mean(x), 2, tau);
    tr2 = embed(y.-mean(y), 2, tau);
    
    
    phase = [atan.(tr1[:,2],tr1[:,1]) atan.(tr2[:,2],tr2[:,1])];
    freq = [diff(phase[:,1]) diff(phase[:,2])]/dt;
    
    idx = findall(phase == 0)
    
    # phase difference
    dphase = broadcast(abs,diff(phase,dims=2));

    return dphase
end

function find(c)
    a = similar(c, Bool)
    count = 1
    @inbounds for i in eachindex(c)
        a[count] = i
        count += (c[i] != zero(eltype(c)))
    end
    return resize!(a, count-1)
end
